# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/00_atyp.ipynb.

# %% auto 0
__all__ = ['G', 'P', 'TypeLike', 'TypeDesc', 'GuardFunc', 'PathLike', 'sbytes', 'notsbytes', 'pdbindex', 'pdindexlike', 'npbool',
           'npnan', 'npNaN', 'npNAN', 'npmatrix', 'npndarray', 'pdna', 'pdindex', 'pdseries', 'pdcindex', 'dataframe',
           'sparr', 'spmtx', 'pttype', 'ptdevice', 'pttensor', 'ptdataset', 'ptdataloader', 'pldatamodule', 'cvvideo',
           'anndata']

# %% ../nbs/00_atyp.ipynb 4
from typing import Type, Union, Callable, ForwardRef, TypeVar, ParamSpec, TypeGuard, TypeAlias

# %% ../nbs/00_atyp.ipynb 6
G = TypeVar('G')
P = ParamSpec('P')

TypeLike:  TypeAlias = Union[Type, TypeAlias, Type[G]]
TypeDesc: TypeAlias = Callable[[TypeLike], str]
GuardFunc: TypeAlias = Callable[P, TypeGuard[TypeLike]]

# %% ../nbs/00_atyp.ipynb 8
try: import os, pathlib
except ImportError: ...

PathLike: TypeAlias = Union[str, pathlib.Path, os.PathLike]

# %% ../nbs/00_atyp.ipynb 11
try: from atyp import (StrBytes, NotStrBytes)
except ImportError: StrBytes, NotStrBytes = type, type
    
sbytes: TypeAlias  = ForwardRef('StrBytes', module='atyp', is_class=True)
notsbytes: TypeAlias  = ForwardRef('NotStrBytes', module='atyp', is_class=True)

# %% ../nbs/00_atyp.ipynb 13
try: from atyp import (BoolIndex, IndexLike)
except ImportError: BoolIndex, IndexLike = list, list

pdbindex: TypeAlias  = ForwardRef('BoolIndex', module='atyp', is_class=True)
pdindexlike: TypeAlias  = ForwardRef('IndexLike', module='atyp', is_class=True)

# %% ../nbs/00_atyp.ipynb 15
try: from numpy import bool_
except ImportError: bool_ = bool

npbool: TypeAlias = ForwardRef('np.bool_', module='numpy', is_class=True)

# %% ../nbs/00_atyp.ipynb 16
try: from numpy import (nan, NaN, NAN)
except ImportError: nan, NaN, NAN = None, None, None

npnan: TypeAlias = ForwardRef('np.nan', module='numpy', is_class=False)
npNaN: TypeAlias = ForwardRef('np.NaN', module='numpy', is_class=False)
npNAN: TypeAlias = ForwardRef('np.NAN', module='numpy', is_class=False)

# %% ../nbs/00_atyp.ipynb 17
try: from numpy import (matrix, ndarray)
except ImportError: matrix, ndarray = list, list

npmatrix: TypeAlias   = ForwardRef('np.matrix',  module='numpy', is_class=True)
npndarray: TypeAlias  = ForwardRef('np.ndarray', module='numpy', is_class=True)

# %% ../nbs/00_atyp.ipynb 19
try: import pandas as pd
except ImportError: pd = None

# %% ../nbs/00_atyp.ipynb 20
try: from pandas import (NA)
except ImportError: 
    NA = None

pdna: TypeAlias = ForwardRef('pd.NA', module='pandas')

# %% ../nbs/00_atyp.ipynb 21
try: from pandas import (Index, Series, CategoricalIndex, DataFrame)
except ImportError: Index, Series, CategoricalIndex, DataFrame = list, list, list, list

pdindex: TypeAlias   = ForwardRef('pd.Index',     module='pandas', is_class=True)
pdseries: TypeAlias  = ForwardRef('pd.Series',    module='pandas', is_class=True)
pdcindex: TypeAlias  = ForwardRef('pd.CategoricalIndex', module='pandas', is_class=True)
dataframe: TypeAlias = ForwardRef('pd.DataFrame', module='pandas', is_class=True)

# %% ../nbs/00_atyp.ipynb 23
try: from scipy.sparse import sparray, spmatrix
except ImportError: sparray, spmatrix = list, list

sparr: TypeAlias = ForwardRef('sp.sparse.sparray',  module='scipy.sparse', is_class=True)
spmtx: TypeAlias = ForwardRef('sp.sparse.spmatrix', module='scipy.sparse', is_class=True)

# %% ../nbs/00_atyp.ipynb 25
try: from torch import dtype, device, Tensor
except ImportError: dtype, device, Tensor = type, str, list
    
pttype: TypeAlias = ForwardRef('torch.dtype', module='torch', is_class=True)
ptdevice: TypeAlias = ForwardRef('torch.device', module='torch', is_class=True)
pttensor: TypeAlias = ForwardRef('torch.Tensor', module='torch', is_class=True)

# %% ../nbs/00_atyp.ipynb 26
try: from torch.utils.data import (Dataset, DataLoader)    
except ImportError: Dataset, DataLoader = list, list

ptdataset = ForwardRef('torch.utils.data.Dataset', module='torch.utils.data', is_class=True)
ptdataloader = ForwardRef('torch.utils.data.DataLoader', module='torch.utils.data', is_class=True)

# %% ../nbs/00_atyp.ipynb 28
try: from pytorch_lightning import (LightningDataModule as DataModule)
except ImportError: DataModule = list
    
pldatamodule = ForwardRef('pytorch_lightning.LightningDataModule', module='pytorch_lightning', is_class=True)

# %% ../nbs/00_atyp.ipynb 30
try: from cv2 import VideoCapture
except ImportError: VideoCapture = type    

cvvideo: TypeAlias = ForwardRef('cv2.VideoCapture', module='cv2', is_class=True)

# %% ../nbs/00_atyp.ipynb 32
try: from anndata import AnnData
except ImportError: AnnData = list

anndata: TypeAlias = ForwardRef('ad.AnnData', module='anndata', is_class=True)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/71_biol.ipynb.

# %% auto 0
__all__ = ['isfilteredmatrixpath', 'isadatafile', 'isfilteredmatrixdir', 'isfmatdir', 'isfmat', 'issensitivitypath',
           'issensitivitydir', 'issensdir', 'issens']

# %% ../nbs/71_biol.ipynb 4
from pathlib import Path
from typing import TypeGuard

# %% ../nbs/71_biol.ipynb 5
from .atyp import PathLike

# %% ../nbs/71_biol.ipynb 8
def isfilteredmatrixpath(p: PathLike) -> TypeGuard[PathLike]:
    '''Check if the current path is a filtered matrix path'''
    return Path(p).name == 'filtered_matrix'

# %% ../nbs/71_biol.ipynb 9
def isadatapath(p: PathLike) -> TypeGuard[PathLike]:
    '''Check if the current path is an anndatat file path'''
    name = Path(p).name
    return name == 'adata.h5ad' or 'data.h5' in name

def isbarcodespath(p: PathLike) -> TypeGuard[PathLike]:
    '''Check if the current path is a barcodes file path'''
    return Path(p).name == 'barcodes.tsv'

def isfeaturespath(p: PathLike) -> TypeGuard[PathLike]:
    '''Check if the current path is a filtered matrix path'''
    return Path(p).name == 'features.tsv'

def ismatrixpath(p: PathLike) -> TypeGuard[PathLike]:
    '''Check if the current path is a filtered matrix path'''
    return Path(p).name == 'matrix.mtx'

# %% ../nbs/71_biol.ipynb 10
def isbarcodesfile(p: PathLike) -> TypeGuard[PathLike]:
    '''Check if the current path is a barcodes file'''
    return isbarcodespath(p) and Path(p).is_file()

def isfeaturesfile(p: PathLike) -> TypeGuard[PathLike]:
    '''Check if the current path is a features file'''
    return isfeaturespath(p) and Path(p).is_file()

def ismatrixfile(p: PathLike) -> TypeGuard[PathLike]:
    '''Check if the current path is a matrix file'''
    return ismatrixpath(p) and Path(p).is_file()

# %% ../nbs/71_biol.ipynb 11
def isadatafile(p: PathLike) -> TypeGuard[PathLike]:
    '''Check if the current path is an adata file'''
    return isadatapath(p) and Path(p).is_file()

# %% ../nbs/71_biol.ipynb 12
def isfilteredmatrixdir(p: PathLike, checkname: bool = True, checkfiles: bool = False) -> TypeGuard[PathLike]:
    '''Check if the current path is a filtered matrix path

    Parameters
    ----------
    p : PathLike
        Path to check
    checkname : bool, optional
        Check if the path name is `filtered_matrix`, by default True
    checkfiles : bool, optional
        Check if the path contains the required files, by default False
    
    See Also
    --------
    isfmat : Alias for `isfilteredmatrixdir`.
    isfmatdir : Alias for `isfilteredmatrixdir`.
    '''
    p = Path(p)
    isnamedir = isfilteredmatrixpath(p) and p.is_dir()
    has_files = (
        isbarcodespath(p / 'barcodes.tsv') and 
        isfeaturespath(p / 'features.tsv') and 
        ismatrixpath(p / 'matrix.mtx')
    )
    return (checkname and isnamedir) or (checkfiles and has_files)

def isfmatdir(p: PathLike, checkname: bool = True, checkfiles: bool = True) -> TypeGuard[PathLike]:
    '''Check if the current path is a filtered matrix path
    
    Parameters
    ----------
    p : PathLike
        Path to check
    checkname : bool, optional
        Check if the path name is `filtered_matrix`, by default True
    checkfiles : bool, optional
        Check if the path contains the required files, by default False

    See Also
    --------
    isfilteredmatrixdir : Alias for `isfmatdir`.
    '''
    return isfilteredmatrixdir(p, checkname, checkfiles)


def isfmat(p: PathLike, checkname: bool = True, checkfiles: bool = True) -> TypeGuard[PathLike]:
    '''Check if the current path is a filtered matrix path
    
    Parameters
    ----------
    p : PathLike
        Path to check
    checkname : bool, optional
        Check if the path name is `filtered_matrix`, by default True
    checkfiles : bool, optional
        Check if the path contains the required files, by default False

    See Also
    --------
    isfilteredmatrixdir : Alias for `isfmat`.
    '''
    return isfilteredmatrixdir(p, checkname, checkfiles)

# %% ../nbs/71_biol.ipynb 13
def issensitivitypath(p: PathLike) -> TypeGuard[PathLike]:
    '''Check if the current path is a sensitivity path'''
    return Path(p).name.startswith('sensitivity')

def issensitivitydir(
    p: PathLike,
    checkname: bool = True, checkfiles: bool = False, checkparent: bool = False
) -> TypeGuard[PathLike]:
    '''Check if the current path is a sensitivity dir

    Parameters
    ----------
    p : PathLike
        Path to check
    checkname : bool, default=True
        Check if the path name has `sensitivity`, by default True
    checkfiles : bool, default=False
        Check if the path contains the required files, by default False
    checkparent : bool, default=False
        Check if the path parent is `filtered_matrix`, by default False
    
    See Also
    --------
    issens : Alias for `issensitivitydir`.
    issensdir : Alias for `issensitivitydir`.
    '''
    p = Path(p)
    if not (checkfiles and checkparent): checkname = True
    sname = issensitivitypath(p) and p.is_dir()
    files = isfilteredmatrixdir(p, checkname=False, checkfiles=checkfiles)
    fpath = isfilteredmatrixdir(p, checkname=True, checkfiles=False)
    return (checkname and sname) or (checkfiles and files) or (checkparent and fpath)


def issensdir(p: PathLike, checkname: bool = True, checkfiles: bool = False, checkparent: bool = False) -> TypeGuard[PathLike]:
    '''Check if the current path is a sensitivity dir

    Parameters
    ----------
    p : PathLike
        Path to check
    checkname : bool, default=True
        Check if the path name has `sensitivity`, by default True
    checkfiles : bool, default=False
        Check if the path contains the required files, by default False
    checkparent : bool, default=False
        Check if the path parent is `filtered_matrix`, by default False
    
    See Also
    --------
    issensitivitydir : Alias for `issensdir`.
    '''
    return issensitivitydir(p, checkname, checkfiles, checkparent)

def issens(
    p: PathLike,
    checkname: bool = True, checkfiles: bool = False, checkparent: bool = False
) -> TypeGuard[PathLike]:
    '''Check if the current path is a sensitivity dir

    Parameters
    ----------
    p : PathLike
        Path to check
    checkname : bool, default=True
        Check if the path name has `sensitivity`, by default True
    checkfiles : bool, default=False
        Check if the path contains the required files, by default False
    checkparent : bool, default=False
        Check if the path parent is `filtered_matrix`, by default False
    
    See Also
    --------
    issensitivitydir : Alias for `issens`.
    '''
    return issensitivitydir(p, checkname, checkfiles, checkparent)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/13_call.ipynb.

# %% auto 0
__all__ = ['isattrcall', 'iscallattr', 'hasget', 'haskeys']

# %% ../nbs/13_call.ipynb 4
from typing import TypeGuard
from typing import Callable

# %% ../nbs/13_call.ipynb 5
from .core import iscall
from .atyp import GuardFunc

# %% ../nbs/13_call.ipynb 7
def isattrcall(attr: str) -> GuardFunc:
    '''Checks if the attribute of the object is a callable'''
    def decorator(obj) -> GuardFunc:
        return iscall(getattr(obj, attr, None))
    return decorator

def iscallattr(obj, attr: str) -> TypeGuard[Callable]:
    '''Checks if `obj.attr` is a callable'''
    return isattrcall(attr)(obj)

# %% ../nbs/13_call.ipynb 8
def hasget(obj) -> TypeGuard[Callable]:
    '''Checks if `obj.get` is a callable'''
    return iscallattr(obj, 'get')

def haskeys(obj) -> TypeGuard[Callable]: 
    '''Checks if `obj.keys` is a callable'''
    return iscallattr(obj, 'keys')



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/01_core.ipynb.

# %% auto 0
__all__ = ['isclass', 'iscls', 'isint', 'isstr', 'isset', 'isnum', 'isbool', 'iscall', 'isdict', 'islist', 'isiter', 'isnone',
           'notnone', 'isany', 'istype', 'istuple', 'istup', 'isslice', 'isslc', 'isrange', 'isrng', 'isbytes']

# %% ../nbs/01_core.ipynb 4
import inspect
from numbers import Number
from typing import TypeGuard

from typing import Any, Type, Callable, Iterable

# %% ../nbs/01_core.ipynb 6
def isclass(x: Type) -> TypeGuard[Type]:
    '''Check if `x` is a `class`.
    
    See Also
    --------
    iscls : Alias for `isclass`.
    '''
    return inspect.isclass(x)

def iscls(x: Type) -> TypeGuard[Type]:
    '''Check if `x` is a `class`.
    
    See Also
    --------
    isclass : Check if `x` is a `class`.
    '''
    return isclass(x)

def isint(x) -> TypeGuard[int]:
    '''Check if `x` is an `int`.'''
    return isinstance(x, int)

def isstr(x) -> TypeGuard[str]:
    '''Check if `x` is an `str`.'''
    return isinstance(x, str)

def isset(x) -> TypeGuard[set]:
    '''Check if `x` is a `set`.'''
    return isinstance(x, set)

def isnum(x) -> TypeGuard[Number]: 
    '''Check if `x` is a `Number`.'''
    return isinstance(x, Number)

# %% ../nbs/01_core.ipynb 7
def isbool(x) -> TypeGuard[bool]:
    '''Check if `x` is `bool`.'''
    return isinstance(x, bool)

def iscall(x) -> TypeGuard[Callable]:
    '''Check if `x` is `Callable`.'''
    return isinstance(x, Callable)

def isdict(x) -> TypeGuard[dict]:
    '''Check if `x` is a `dict`.'''
    return isinstance(x, dict)

def islist(x) -> TypeGuard[list]:
    '''Check if `x` is a `list`.'''
    return isinstance(x, list)

def isiter(x) -> TypeGuard[Iterable]:
    '''Check if `x` is an `Iterable`.'''
    return isinstance(x, Iterable)

def isnone(x) -> TypeGuard[None]:
    '''Check if `x` is `None`.'''
    return x is None

def notnone(x) -> TypeGuard[Any]:
    '''Check if `x` is not `None`.
    
    See Also
    --------
    isany : alias for `notnone`.
    '''
    return not isnone(x)

def isany(x) -> TypeGuard[Any]:
    '''Check if `x` is not `None`.
    
    See Also
    --------
    notnone : alias for `isany`.
    '''
    return notnone(x)


def istype(x) -> TypeGuard[type]:
    '''Check if `x` is `type`.'''
    return isinstance(x, type)


# %% ../nbs/01_core.ipynb 8
def istuple(x) -> TypeGuard[tuple]:
    '''Check if `x` is an `tuple`.
    
    See Also
    --------
    istup : alias for `istuple`.
    '''
    return isinstance(x, tuple)

def istup(x) -> TypeGuard[tuple]:
    '''Check if `x` is an `tuple`.
    
    See Also
    --------
    istuple : alias for `istup`.
    '''
    return istuple(x)

# %% ../nbs/01_core.ipynb 9
def isslice(x) -> TypeGuard[slice]:
    '''Check if `x` is an `slice`.
    
    See Also
    --------
    isslc : alias for `isslice`.
    '''
    return isinstance(x, slice)

def isslc(x) -> TypeGuard[slice]:
    '''Check if `x` is an `slice`.
    
    See Also
    --------
    isslice : alias for `isslc`.
    '''
    return isslice(x)

# %% ../nbs/01_core.ipynb 10
def isrange(x) -> TypeGuard[range]:
    '''Check if `x` is a `range`.
    
    See Also
    --------
    isrng : alias for `isrange`.
    '''
    return isinstance(x, range)

def isrng(x) -> TypeGuard[range]:
    '''Check if `x` is a `range`.
    
    See Also
    --------
    isrange : alias for `isrng`.
    '''
    return isrange(x)

# %% ../nbs/01_core.ipynb 11
def isbytes(x) -> TypeGuard[bytes]:
    '''Check if `x` is an `bytes`.'''
    return isinstance(x, bytes)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/14_bool.ipynb.

# %% auto 0
__all__ = ['is01', 'isbooly', 'isbooltype', 'isbtype', 'isboolcall', 'isbcall']

# %% ../nbs/14_bool.ipynb 4
from typing import TypeGuard
from typing import Type, Callable, Literal, Union

# %% ../nbs/14_bool.ipynb 5
from .atyp import bool_, npbool
from .core import isbool, iscall

# %% ../nbs/14_bool.ipynb 7
def is01(x) -> TypeGuard[Union[Literal[0], Literal[1]]]: 
    '''Check if `x` is either `0` or `1`.'''
    return x in [0, 1]

def isbooly(x) -> TypeGuard[Union[bool, npbool, Literal[0], Literal[1]]]:
    '''Check if `x` is one of `bool`, `np.bool_`, `0`, or `1`.'''
    return isbool(x) or is01(x) or isinstance(x, bool_)


def isbooltype(x) -> TypeGuard[Type[bool]]:
    '''Check if `x` is the `bool` type.
    
    See Also
    --------
    isbtype : Alias for `isbooltype`.
    '''
    return x == bool

def isbtype(x) -> TypeGuard[Type[bool]]:
    '''Check if `x` is the `bool` type.
    
    See Also
    --------
    isbooltype : Alias for `isbtype`.
    '''
    return isbooltype(x)

def isboolcall(x) -> TypeGuard[Union[bool, npbool, Literal[0], Literal[1], Callable]]:
    '''Check if `x` is one of `bool`, `np.bool_`, `0`, `1`, or `Callable`.
    
    See Also
    --------
    isbcall : Alias for `isboolcall`.
    '''
    return isbool(x) or iscall(x) or is01(x) or isinstance(x, bool_)

def isbcall(x):
    '''Check if `x` is one of `bool`, `np.bool_`, `0`, `1`, or `Callable`.
    
    See Also
    --------
    isboolcall : Alias for `isbcall`.
    '''
    return isboolcall(x)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/03_tgrd.ipynb.

# %% auto 0
__all__ = ['istypeq', 'isinst', 'isinstq', 'isof']

# %% ../nbs/03_tgrd.ipynb 4
from types import NoneType
from typing import TypeGuard
from typing import Tuple, Union

# %% ../nbs/03_tgrd.ipynb 5
from .atyp import G, TypeLike

# %% ../nbs/03_tgrd.ipynb 7
def istypeq(x: G, t: TypeLike) -> TypeGuard[Union[G, NoneType]]:
    '''Check if `x` is of type `t` or `NoneType`'''
    return isinstance(x, (t, NoneType))

def isinst(x: G, t: TypeLike) -> TypeGuard[G]:
    '''Alias for `isinstance`'''
    return isinstance(x, t)

def isinstq(x: G, t: TypeLike) -> TypeGuard[Union[G, NoneType]]:
    '''Alias for `isinstance`'''
    return isinstance(x, (t, NoneType))

def isof(x: G, t: Tuple[TypeLike, ...]) -> TypeGuard[TypeLike]:
    '''Alias for `isinstance`'''
    return isinst(x, t)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/62_issp.ipynb.

# %% auto 0
__all__ = ['issparray', 'isspmatrix', 'issparr', 'isspmtx', 'issparse', 'isspr']

# %% ../nbs/62_issp.ipynb 4
from typing import TypeGuard
from typing import Union

# %% ../nbs/62_issp.ipynb 5
from .atyp import sparray, spmatrix, sparr, spmtx

# %% ../nbs/62_issp.ipynb 7
def issparray(x) -> TypeGuard[sparr]:
    '''Check if `x` is an `sp.sparse.sparray`.
    
    See Also
    --------
    issparr : Alias for `issparray`.'''
    return isinstance(x, sparray)

def isspmatrix(x) -> TypeGuard[spmtx]:
    '''Check if `x` is an `sp.sparse.spmatrix`.
    
    See Also
    --------
    isspmtx : Alias for `isspmatrix`.
    '''
    return isinstance(x, spmatrix)

def issparr(x) -> TypeGuard[sparr]:
    '''Check if `x` is an `sp.sparse.sparray`.
    
    See Also
    --------
    issparray : Alias for `issparr`.
    '''
    return issparray(x)

def isspmtx(x) -> TypeGuard[spmtx]:
    '''Check if `x` is an `sp.sparse.spmatrix`.
    
    See Also
    --------
    isspmatrix : Alias for `isspmtx`.
    '''
    return isspmatrix(x)

def issparse(x) -> TypeGuard[Union[sparr, spmtx]]:
    '''Check if `x` is an `sp.sparse.sparray` or `sp.sparse.spmatrix`.
    
    See Also
    --------
    issparray : Alias for `issparr`.
    isspmatrix : Alias for `isspmtx`.
    '''
    return issparray(x) or isspmatrix(x)

def isspr(x) -> TypeGuard[Union[sparr, spmtx]]:
    '''Check if `x` is an `sp.sparse.sparray` or `sp.sparse.spmatrix`.
    
    See Also
    --------
    issparray : Alias for `issparr`.
    isspmatrix : Alias for `isspmtx`.
    issparse : Alias for `issparse`.
    '''
    return issparray(x) or isspmatrix(x)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/99_misc.ipynb.

# %% auto 0
__all__ = ['isin', 'arein', 'isna', 'notna']

# %% ../nbs/99_misc.ipynb 4
from typing import TypeGuard
from typing import Any, Iterable

# %% ../nbs/99_misc.ipynb 5
from .atyp import pd, pdna
from .core import isiter, isnone, notnone
from .isnp import isnpnan
from .ispd import ispdna

# %% ../nbs/99_misc.ipynb 7
def isin(x, refs: Iterable) -> bool: return x in refs
def arein(x: Iterable, refs: Iterable) -> bool:
    return isiter(x) and isiter(refs) and all(isin(y, refs) for y in x)

def isna(x) -> TypeGuard[pdna]:
    if notnone(pd): return pd.isna(x)
    return isnone(x) or isnpnan(x) or ispdna(x)

def notna(x) -> TypeGuard[Any]:
    if notnone(pd): return pd.notna(x)
    return not isna(x)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/32_pyty.ipynb.

# %% auto 0
__all__ = ['ismethodwrapper', 'ismthdwrap', 'isbuiltinfunctiontype', 'isbuiltin']

# %% ../nbs/32_pyty.ipynb 4
from types import MethodWrapperType, BuiltinFunctionType
from typing import TypeGuard

# %% ../nbs/32_pyty.ipynb 6
def ismethodwrapper(x) -> TypeGuard[MethodWrapperType]:
    '''Check if x is of type `MethodWrapperType`.
    
    See Also
    --------
    ismthdwrap: alias for `ismethodwrapper`    
    '''
    return isinstance(x, MethodWrapperType)


def ismthdwrap(x) -> TypeGuard[MethodWrapperType]:
    '''Check if x is of type `MethodWrapperType`.
    
    See Also
    --------
    ismethodwrapper: alias for `ismethismthdwrapodwrapper`    
    '''
    return ismethodwrapper(x)

def isbuiltinfunctiontype(x) -> TypeGuard[BuiltinFunctionType]:
    '''Check if x is of type `BuiltinFunctionType`.
    
    See Also
    --------
    isbuiltin: alias for `isbuiltinfunctiontype`    
    '''
    return isinstance(x, BuiltinFunctionType)

def isbuiltin(x) -> TypeGuard[BuiltinFunctionType]:
    '''Check if x is of type `BuiltinFunctionType`.
    
    See Also
    --------
    isbuiltinfunctiontype: alias for `isbuiltin`    
    '''
    return isbuiltinfunctiontype(x)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/11_path.ipynb.

# %% auto 0
__all__ = ['ispath', 'isxath', 'ispathlike', 'isdir', 'isfile', 'ishidden', 'ispathy']

# %% ../nbs/11_path.ipynb 4
import os, pathlib
from typing import TypeGuard
from typing import Union, ForwardRef

# %% ../nbs/11_path.ipynb 5
from .atyp import PathLike
from .fref import dereference

# %% ../nbs/11_path.ipynb 7
def ispath(x) -> TypeGuard[pathlib.Path]:
    '''Check if `x` is `pathlib.Path`.'''
    return isinstance(x, pathlib.Path)

def isxath(
    x,
    globalsns: dict = None, localsns: dict = None, recursive_guard: set = None
) -> TypeGuard['Xath']:
    '''Check if `x` is `Xath`.'''
    xref = ForwardRef('Xath', module='xath', is_class=True)
    xtyp = dereference(xref, globalsns=globalsns, localsns=localsns, recursive_guard=recursive_guard)
    return isinstance(x, xtyp)

def ispathlike(x) -> TypeGuard[PathLike]:
    '''Check if `x` is `str`, `pathlib.Path` or `os.PathLike`.'''
    return isinstance(x, (str, pathlib.Path, os.PathLike))

def isdir(x) -> TypeGuard[pathlib.Path]: 
    '''Check if `x` is `pathlib.Path` and exists.'''
    return ispathlike(x) and pathlib.Path(x).is_dir()

def isfile(x) -> TypeGuard[pathlib.Path]: 
    '''Check if `x` is `pathlib.Path` and exists.'''
    return ispathlike(x) and pathlib.Path(x).is_file()

def ishidden(x) -> TypeGuard[pathlib.Path]: 
    '''Check if `x` is `pathlib.Path` and is a hidden file or directory.'''
    pname = pathlib.Path(x).name
    return ispathlike(x) and (pname.startswith('.') or pname.startswith('_'))

def ispathy(x) -> TypeGuard[Union['Xath', PathLike]]:
    '''Check if `x` is `str`, `pathlib.Path`, `os.PathLike`, or `Xath`.'''
    return isxath(x) and ispathlike(x)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/10_strs.ipynb.

# %% auto 0
__all__ = ['isallstr', 'isprivate', 'ismangled', 'issunder', 'isdunder', 'isnilstr', 'ispublic', 'isstar', 'iselps', 'ispipe',
           'ispunc', 'isupper', 'islower', 'isquote', 'ischar', 'ischars', 'isstrbytes', 'notstrbytes', 'isuniqchrs',
           'isdig', 'isdotstr', 'ispipestr']

# %% ../nbs/10_strs.ipynb 4
import string
from typing import TypeGuard

from typing import Union, Literal, Iterable

# %% ../nbs/10_strs.ipynb 5
from .core import isiter, isstr, isbytes

# %% ../nbs/10_strs.ipynb 7
def isallstr(x) -> TypeGuard[Iterable[str]]:
    '''Check if `x` is an `Iterable[str]`.'''
    return isiter(x) and all(isstr(e) for e in x)

# %% ../nbs/10_strs.ipynb 9
def isprivate(s: str) -> TypeGuard[str]:
    '''Check if `s` is a private variable name e.g. `'_{anything}'`.'''
    return isstr(s) and s.startswith('_') and len(s) > 1

def ismangled(s: str) -> TypeGuard[str]:
    '''Check if `s` is an mangled variable name e.g. `'__{anything}'`.'''
    return isstr(s) and s.startswith('__') and len(s) > 2

def issunder(s: str) -> TypeGuard[str]:
    '''Check if `s` is a sunder e.g. `'_{anything}_'`.'''
    return isstr(s) and s.startswith('_') and s.endswith('_') and len(s) > 2

def isdunder(s: str) -> TypeGuard[str]:
    '''Check if `s` is an dunder e.g. `'__{anything}__'`.'''
    return isstr(s) and s.startswith('__') and s.endswith('__') and len(s) > 4

def isnilstr(s: str) -> TypeGuard[Literal['']]:
    '''Check if `s` is the nil string `''`.'''
    return isstr(s) and len(s) == 0

def ispublic(s: str) -> TypeGuard[str]:
    '''Check if `s` is a public variable name e.g. `'{anything}'`.'''
    return isstr(s) and not isprivate(s) and not ismangled(s) and not issunder(s) and not isdunder(s)

# %% ../nbs/10_strs.ipynb 10
def isstar(s: str) -> TypeGuard[Literal['*']]:
    '''Check if `s` is an asterick string `'*'`.'''
    return isstr(s) and s == '*'

def iselps(s: str) -> TypeGuard[Literal['...']]:
    '''Check if `s` is an ellipse string `...''`.'''
    return isstr(s) and s == '...'


def ispipe(s: str) -> TypeGuard[Literal['|']]:
    '''Check if `s` is a pipe string `'|'`.'''
    return isstr(s) and s == '|'

# %% ../nbs/10_strs.ipynb 11
def ispunc(s: str) -> TypeGuard[str]:
    return isstr(s) and s in string.punctuation

def isupper(s: str) -> TypeGuard[str]:
    return isstr(s) and s in string.ascii_uppercase

def islower(s: str) -> TypeGuard[str]:
    return isstr(s) and s in string.ascii_lowercase

# %% ../nbs/10_strs.ipynb 12
def isquote(s: str) -> TypeGuard[Literal['"', "'"]]:
    '''Check if `s` is a single or double quote string `'` or `"`.'''
    return isstr(s) and ispunc(s) and s in {'"', "'"}

# %% ../nbs/10_strs.ipynb 13
def ischar(s: str, char: str = '') -> TypeGuard[str]:
    '''Checks if a `s` is `char`.'''
    return isstr(s) and isstr(char) and s == char

def ischars(s: str, chars: str = string.punctuation) -> TypeGuard[str]:
    '''Checks if a string only contains characters from a set of valid characters.'''
    return all(c in chars for c in s)

# %% ../nbs/10_strs.ipynb 14
def isstrbytes(s: str) -> TypeGuard[Union[str, bytes]]:
    '''Checks if a `s` is either `str` or `bytes`.'''
    return isstr(s) or isbytes(s)

def notstrbytes(s: str) -> TypeGuard[Union[str, bytes]]:
    '''Checks if a `s` is neither `str` or `bytes`.'''
    return not isstrbytes(s)

# %% ../nbs/10_strs.ipynb 15
def isuniqchrs(s: str) -> bool: # isuniq
    '''Checks if a string only contains unique characters.'''
    return len(set(s)) == len(s)

# %% ../nbs/10_strs.ipynb 16
def isdig(s: str) -> TypeGuard[str]:
    '''Check if `s` is a digit character'''
    return isstr(s) and str.isdigit(s)

def isdotstr(s: str) -> TypeGuard[str]:
    '''Check if `s` is a dot str e.g. `'3.10.12.410'`'''
    return isstr(s) and all(isdig(e) for e in s.split('.'))    

def ispipestr(s: str) -> TypeGuard[str]:
    '''Check if `s` contains a pipe.'''
    return isstr(s) and any(map(ispipe, s))



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/10_isad.ipynb.

# %% auto 0
__all__ = ['anndata', 'isanndata', 'isad']

# %% ../nbs/10_isad.ipynb 4
from typing import TypeGuard, ForwardRef, TypeAlias
try: from anndata import AnnData
except ImportError: AnnData = list
anndata: TypeAlias = ForwardRef('ad.AnnData', module='anndata', is_class=True)

# %% ../nbs/10_isad.ipynb 6
def isanndata(x) -> TypeGuard[anndata]:
    '''Check if `x` is an `ad.AnnData`.'''
    return isinstance(x, AnnData)

def isad(x) -> TypeGuard[anndata]:
    '''Check if `x` is an `ad.AnnData`.'''
    return isinstance(x, AnnData)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/02_comp.ipynb.

# %% auto 0
__all__ = ['islistordict']

# %% ../nbs/02_comp.ipynb 4
from typing import TypeGuard
from typing import Any, Union

# %% ../nbs/02_comp.ipynb 5
from .core import islist, isdict

# %% ../nbs/02_comp.ipynb 7
def islistordict(x: Any) -> TypeGuard[Union[list, dict]]:
    '''
    Check if `x` is a `list` or `dict`.

    Parameters
    ----------
    x : Any
        The variable to check.

    Returns
    -------
    TypeGuard[Union[list, dict]
        True if `x` is a `list` or `dict`, False otherwise.
    '''
    return islist(x) or isdict(x)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/61_ispd.ipynb.

# %% auto 0
__all__ = ['ispdna', 'notpdna', 'isdf', 'isidx', 'isseries', 'iscategoricalindex', 'iscatidx', 'iscidx', 'isboolindex',
           'isboolidx', 'isbidx', 'isindexlike', 'isidxlike']

# %% ../nbs/61_ispd.ipynb 4
from typing import TypeGuard
from typing import Any, Optional

# %% ../nbs/61_ispd.ipynb 5
from .atyp import (NA, DataFrame, Index, Series, CategoricalIndex)
from .atyp import (pdna, dataframe, pdindex, pdseries, pdcindex)
from .atyp import (BoolIndex, IndexLike, pdbindex, pdindexlike)

from .core import notnone

# %% ../nbs/61_ispd.ipynb 7
def ispdna(x) -> TypeGuard[pdna]:
    '''Check if `x` is `pd.NA`.'''
    return x is NA

def notpdna(x) -> TypeGuard[Any]:
    '''Check if `x` is not `pd.NA`.'''
    return not ispdna(x)

# %% ../nbs/61_ispd.ipynb 8
def isdf(x) -> TypeGuard[dataframe]:
    '''Check if `x` is an `pd.DataFrame`.'''
    return isinstance(x, DataFrame)

def isidx(x) -> TypeGuard[pdindex]:
    '''Check if `x` is an `pd.Index`.'''
    return isinstance(x, Index)

def isseries(x) -> TypeGuard[pdseries]:
    '''Check if `x` is an `pd.Series`.'''
    return isinstance(x, Series)

# %% ../nbs/61_ispd.ipynb 9
def iscategoricalindex(x) -> TypeGuard[pdcindex]:
    '''Check if `x` is an `pd.CategoricalIndex`.    
    
    See Also
    --------
    iscatidx : Alias for `iscategoricalindex`.
    iscidx : Alias for `iscategoricalindex`.
    '''
    return isinstance(x, CategoricalIndex)

def iscatidx(x) -> TypeGuard[pdcindex]:
    '''Check if `x` is an `pd.CategoricalIndex`.    
    
    See Also
    --------
    iscatidx : Alias for `iscatidx`.
    iscidx: Alias for `iscatidx`.
    '''
    return iscategoricalindex(x)

def iscidx(x) -> TypeGuard[pdcindex]:
    '''Check if `x` is an `pd.CategoricalIndex`.    
    
    See Also
    --------
    iscatidx : Alias for `iscidx`.
    iscategoricalindex: Alias for `iscidx`.
    '''
    return iscategoricalindex(x)


def isboolindex(x, r: Optional[pdindex] = None) -> TypeGuard[pdbindex]:
    '''Check if `x` is an `BoolIndex`. 

    Parameters
    ----------
    x : `Any`
        The object to check.

    r : `pd.Index`, default: None
        An optional reference index to check that `x` is the same length of.
    
    See Also
    --------
    isboolidx : Alias for `isboolindex`.
    isbidx : Alias for `isboolindex`.
    '''
    return isinstance(x, BoolIndex) and (len(x) == len(r) if notnone(r) else True)

def isboolidx(x, r: Optional[pdindex] = None) -> TypeGuard[pdbindex]:
    '''Check if `x` is an `BoolIndex`.    

    Parameters
    ----------
    x : `Any`
        The object to check.
        
    r : `pd.Index`, default: None
        An optional reference index to check that `x` is the same length of.
    
    See Also
    --------
    isboolindex : Alias for `isboolidx`.
    isbidx: Alias for `isboolidx`.
    '''
    return isboolindex(x, r)

def isbidx(x, r: Optional[pdindex] = None) -> TypeGuard[pdbindex]:
    '''Check if `x` is an `BoolIndex`.    
    Parameters
    ----------
    x : `Any`
        The object to check.
        
    r : `pd.Index`, default: None
        An optional reference index to check that `x` is the same length of.
    
    See Also
    --------
    isboolidx : Alias for `isbidx`.
    isboolindex: Alias for `isbidx`.
    '''
    return isboolindex(x, r)



def isindexlike(x) -> TypeGuard[pdindexlike]:
    '''Check if `x` is an `IndexLike`.    
    
    See Also
    --------
    isidxlike : Alias for `isindexlike`.
    '''
    return isinstance(x, IndexLike)

def isidxlike(x) -> TypeGuard[pdindexlike]:
    '''Check if `x` is an `IndexLike`.    
    
    See Also
    --------
    isindexlike : Alias for `isidxlike`.
    '''
    return isindexlike(x)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/04_wrap.ipynb.

# %% auto 0
__all__ = ['vanilla', 'isguard', 'guardname', 'guarddocs', 'guardanns', 'setwrapattrs', 'makeguard', 'wrapguard', 'decoguard']

# %% ../nbs/04_wrap.ipynb 4
from functools import wraps

from typing import TypeGuard
from typing import Any, Type

# %% ../nbs/04_wrap.ipynb 5
from .atyp import G, P, TypeDesc, GuardFunc

# %% ../nbs/04_wrap.ipynb 7
def guardname(cls: G):
    return f'is{cls.__name__.lower()}'

def guarddocs(cls: G) -> str:
    return f'''Check if `x` is a `{cls.__name__}`.'''

def guardanns(cls: G) -> dict:
    return {'x': 'typing.Any', 'return': TypeGuard[Type[cls]]}

def setwrapattrs(
    call: GuardFunc, cls: G,
    docsfn: TypeDesc = guarddocs,
    namefn: TypeDesc = guardname,
    annsfn: TypeDesc = guardanns,
) -> GuardFunc:
    # Update docstring
    call.__doc__ = call.__doc__ or docsfn(cls)    
    # Update name
    call.__name__ = namefn(cls)
    # Update annotations
    call.__annotations__ = annsfn(cls)
    return call

# %% ../nbs/04_wrap.ipynb 9
def makeguard(cls: G) -> GuardFunc:
    def guard(x: Any) -> TypeGuard[cls]:
        f'''Check if `x` is a `{cls.__name__}`.'''        
        return isinstance(x, cls)
    guard = setwrapattrs(guard, cls)
    return guard

def wrapguard(cls: G, call: GuardFunc) -> GuardFunc:
    @wraps(call)
    def wrapper(x: Any, *args: P.args, **kwargs: P.kwargs) -> TypeGuard[Type[cls]]:
        f'''Check if `x` is a `{cls.__name__}`.'''
        res = None
        if kwargs.get('__call_wrapped__', True): res = call(x)
        if kwargs.get('__pass_result__', False): x = res
        return isinstance(x, cls)
        
    for mthd in (wrapper, call): mthd = setwrapattrs(mthd, cls)
    return wrapper

# %% ../nbs/04_wrap.ipynb 11
def decoguard(cls: G) -> GuardFunc:
    '''Automatically adds a type guard and docstring to a function.'''
    def decorator(call: GuardFunc) -> GuardFunc:
        return wrapguard(cls, call)
    return decorator

# %% ../nbs/04_wrap.ipynb 13
vanilla = makeguard
isguard = decoguard



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/91_lens.ipynb.

# %% auto 0
__all__ = ['islenn', 'islen1', 'islen2', 'islen3', 'islen4', 'hasn', 'has1', 'has2', 'has3', 'has4', 'isnints', 'is2ints',
           'is3ints', 'is4ints', 'ischr', 'ischar']

# %% ../nbs/91_lens.ipynb 4
from typing import TypeGuard
from typing import Iterable, Optional

# %% ../nbs/91_lens.ipynb 5
from .core import isiter, isstr
from .alls import isints

# %% ../nbs/91_lens.ipynb 7
def islenn(a, n: int = 1) -> TypeGuard[Iterable]:
    '''Check if `a` has exactly `n` elements.'''
    return len(a) == n

def islen1(a) -> TypeGuard[Iterable]:
    '''Check if `a` has exactly `1` elements.
    
    See Also
    --------
    islenn : check if arguments contain at n elements.
    '''
    return islenn(a, 1)

def islen2(a) -> TypeGuard[Iterable]:
    '''Check if `a` has exactly `2` elements.
    
    See Also
    --------
    islenn : check if arguments contain at n elements.
    '''
    return islenn(a, 2)

def islen3(a) -> TypeGuard[Iterable]:
    '''Check if `a` has exactly `3` elements.
    
    See Also
    --------
    islenn : check if arguments contain at n elements.
    '''
    return islenn(a, 3)

def islen4(a) -> TypeGuard[Iterable]:
    '''Check if `a` has exactly `4` elements.
    
    See Also
    --------
    islenn : check if arguments contain at n elements.
    '''
    return islenn(a, 4)


# %% ../nbs/91_lens.ipynb 8
def hasn(a, n: int = 1) -> TypeGuard[Iterable]:
    '''Check if `a` contain at least `n` elements.'''
    return isiter(a) and len(a) >= n

def has1(a) -> TypeGuard[Iterable]: 
    '''Check if `a` contain at least `1` elements.
    
    See Also
    --------
    hasn : check if `a` contain at least `n` elements.
    '''
    return hasn(a, 1)

def has2(a) -> TypeGuard[Iterable]: 
    '''Check if `a` contain at least `2` elements.
    
    See Also
    --------
    hasn : check if `a` contain at least `n` elements.
    '''
    return hasn(a, 2)

def has3(a) -> TypeGuard[Iterable]: 
    '''Check if `a` contain at least `3` elements.
    
    See Also
    --------
    hasn : check if `a` contain at least `n` elements.
    '''
    return hasn(a, 3)

def has4(a) -> TypeGuard[Iterable]: 
    '''Check if `a` contain at least `4` elements.
    
    See Also
    --------
    hasn : check if `a` contain at least `n` elements.
    '''
    return hasn(a, 4)


# %% ../nbs/91_lens.ipynb 9
def isnints(a, n: int) -> TypeGuard[Iterable[int]]: 
    '''Check if `a` contains exactly `n` `int`s.'''
    return isints(a) and islenn(a, n)

def is2ints(a) -> TypeGuard[Iterable[int]]:
    '''Check if `a` contain exactly 2 `int`s.
    
    See Also
    --------
    isnints : Check if `a` contains exactly `n` `int`s.
    '''
    return isnints(a, 2)

def is3ints(a) -> TypeGuard[Iterable[int]]: 
    '''Check if `a` contain exactly 3 `int`s.
    
    See Also
    --------
    isnints : Check if `a` contains exactly `n` `int`s.
    '''
    return isnints(a, 3)

def is4ints(a) -> TypeGuard[Iterable[int]]: 
    '''Check if `a` contain exactly 4 `int`s.
    
    See Also
    --------
    isnints : Check if `a` contains exactly `n` `int`s.
    '''
    return isnints(a, 4)

# %% ../nbs/91_lens.ipynb 10
def ischr(s) -> TypeGuard[str]:
    '''Check if `s` is just a single character'''
    return isstr(s) and islenn(s, 1)

def ischar(s: str, char: Optional[str] = None) -> TypeGuard[str]:
    '''Checks if a `s` is just a single character or is the specific `char`.'''
    return isstr(s) and islenn(s, 1) and isstr(char) and s == char



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/15_dict.ipynb.

# %% auto 0
__all__ = ['isdictkeys', 'isdictvalues', 'isdictitems', 'isdkeys', 'isdvals', 'isditems', 'isdictiter', 'isdicttype', 'isditer']

# %% ../nbs/15_dict.ipynb 4
from _collections_abc import dict_keys, dict_values, dict_items
from typing import TypeGuard
from typing import Union

# %% ../nbs/15_dict.ipynb 6
def isdictkeys(x) -> TypeGuard[dict_keys]:
    '''Check if `x` is a `dict_keys`.
    
    See Also
    --------
    isdkeys : Alias for `isdictkeys`.
    '''
    return isinstance(x, dict_keys)

def isdictvalues(x) -> TypeGuard[dict_values]:
    '''Check if `x` is a `dict_values`.
    
    See Also
    --------
    isdvals : Alias for `isdictvalues`.
    '''
    return isinstance(x, dict_values)

def isdictitems(x) -> TypeGuard[dict_items]:
    '''Check if `x` is a `dict_items`.
    
    See Also
    --------
    isditems : Alias for `isdictitems`.
    '''
    return isinstance(x, dict_items)


def isdkeys(x) -> TypeGuard[dict_keys]:
    '''Check if `x` is a `dict_keys`.
    
    See Also
    --------
    isdictkeys : Alias for `isdkeys`.
    '''
    return isdictkeys(x)

def isdvals(x) -> TypeGuard[dict_values]:
    '''Check if `x` is a `dict_values`.
    
    See Also
    --------
    isdictvalues : Alias for `isdvals`.
    '''
    return isdictvalues(x)

def isditems(x) -> TypeGuard[dict_items]:
    '''Check if `x` is a `dict_items`.
    
    See Also
    --------
    isdictitems : Alias for `isditems`.
    '''
    return isdictitems(x)


def isdictiter(x) -> TypeGuard[Union[dict_keys, dict_values, dict_items]]:
    '''Check if `x` is a `dict_keys`, `dict_values`, or `dict_items`.
    
    See Also
    --------
    isditer : Alias for `isdictiter`.
    isdicttype : Alias for `isdictiter`.
    '''
    return isdictkeys(x) or isdictvalues(x) or isdictitems(x)

def isdicttype(x) -> TypeGuard[Union[dict_keys, dict_values, dict_items]]:
    '''Check if `x` is a `dict_keys`, `dict_values`, or `dict_items`.
    
    See Also
    --------
    isdictiter : Alias for `isdicttype`.
    '''
    return isdictiter(x)


def isditer(x) -> TypeGuard[Union[dict_keys, dict_values, dict_items]]:
    '''Check if `x` is a `dict_keys`, `dict_values`, or `dict_items`.
    
    See Also
    --------
    isdictiter : Alias for `isditer`.
    isdicttype : Alias for `isdictiter`.
    '''
    return isdictiter(x)



__version__ = "0.0.50"
# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/00__init__.ipynb.

# %% auto 0
__all__ = []

# %% ../nbs/00__init__.ipynb 3
from itertools import chain
from chck import (
    atyp, core, comp, tgrd, wrap, fref,    
    strs, path, iter, call, bool, dict, insp, dcls, pyty,
    isnp, ispd, issp, trch, iscv, isan, biol,
    alls, lens, misc,
)

# type aliases
from .atyp import *
# core and composite guards
from .core import *
from .comp import *
# wrappers, decorators, factories for guards
from .tgrd import *
from .wrap import *
# typing checks
from .fref import *

# 10s: python types
from .strs import *
from .path import *
from .iter import *
from .call import *
from .bool import *
from .dict import *

# 30s: python libraries
from .insp import *
from .dcls import *
from .pyty import *

# 60s: external libraries
from .isnp import *
from .ispd import *
from .issp import *
from .trch import *
from .iscv import *

# 70s: biology
from .isan import *
from .biol import *

# 90s: mapped guards, explicit lengths, and misc
from .alls import *
from .lens import *
from .misc import *

# %% ../nbs/00__init__.ipynb 4
__all__ = sorted(set(chain(*[
    mod.__all__ for mod in (
        atyp, core, comp, tgrd, wrap, fref,    
        strs, path, iter, call, bool, dict, insp, dcls, pyty, 
        isnp, ispd, issp, trch, iscv, isan, biol,
        alls, lens, misc,
    )
])))



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/70_isan.ipynb.

# %% auto 0
__all__ = ['isanndata', 'isad']

# %% ../nbs/70_isan.ipynb 4
from typing import TypeGuard

# %% ../nbs/70_isan.ipynb 5
from .atyp import AnnData, anndata

# %% ../nbs/70_isan.ipynb 7
def isanndata(x) -> TypeGuard[anndata]:
    '''Check if `x` is an `ad.AnnData`.'''
    return isinstance(x, AnnData)

def isad(x) -> TypeGuard[anndata]:
    '''Check if `x` is an `ad.AnnData`.'''
    return isinstance(x, AnnData)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/05_fref.ipynb.

# %% auto 0
__all__ = ['isforwardref', 'isfref', 'dereference']

# %% ../nbs/05_fref.ipynb 4
from typing import TypeGuard
from typing import ForwardRef

# %% ../nbs/05_fref.ipynb 5
from .core import isstr, isnone

# %% ../nbs/05_fref.ipynb 7
def isforwardref(x) -> TypeGuard[ForwardRef]:
    '''Check if `x` is a `ForwardRef`
    
    See Also
    --------
    isfref : Alias for `isforwardref`.
    '''
    return isinstance(x, ForwardRef)

def isfref(x) -> TypeGuard[ForwardRef]:
    '''Check if `x` is a `ForwardRef`
    
    See Also
    --------
    isforwardref : Alias for `isfref`.
    '''
    return isforwardref(x)

# %% ../nbs/05_fref.ipynb 8
def dereference(
    ref: ForwardRef | str,
    is_argument: bool = False,
    module: str = None,
    *,
    is_class: bool = False,
    globalsns: dict = None, 
    localsns: dict = None, 
    recursive_guard: set = None
) -> type:
    '''Dereference a `ForwardRef` object.'''
    if not isforwardref(ref) and isstr(ref):
        ref = ForwardRef(ref, is_argument=is_argument, module=module, is_class=is_class)

    recursive_guard = set()     if isnone(recursive_guard) else recursive_guard
    localsns        = locals()  if isnone(localsns)        else localsns
    globalsns       = globals() if isnone(globalsns)       else globalsns
    val = ref._evaluate(globalsns, localsns, recursive_guard)
    return val



# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/chck',
                'doc_host': 'https://dsm-72.github.io',
                'git_url': 'https://github.com/dsm-72/chck',
                'lib_path': 'chck'},
  'syms': { 'chck.alls': { 'chck.alls.alldig': ('alls.html#alldig', 'chck/alls.py'),
                           'chck.alls.allint': ('alls.html#allint', 'chck/alls.py'),
                           'chck.alls.allmap': ('alls.html#allmap', 'chck/alls.py'),
                           'chck.alls.allsame': ('alls.html#allsame', 'chck/alls.py'),
                           'chck.alls.allstr': ('alls.html#allstr', 'chck/alls.py'),
                           'chck.alls.alltype': ('alls.html#alltype', 'chck/alls.py'),
                           'chck.alls.areinstances': ('alls.html#areinstances', 'chck/alls.py'),
                           'chck.alls.isints': ('alls.html#isints', 'chck/alls.py'),
                           'chck.alls.isstrs': ('alls.html#isstrs', 'chck/alls.py')},
            'chck.atyp': {},
            'chck.biol': { 'chck.biol.isadatafile': ('biol.html#isadatafile', 'chck/biol.py'),
                           'chck.biol.isadatapath': ('biol.html#isadatapath', 'chck/biol.py'),
                           'chck.biol.isbarcodesfile': ('biol.html#isbarcodesfile', 'chck/biol.py'),
                           'chck.biol.isbarcodespath': ('biol.html#isbarcodespath', 'chck/biol.py'),
                           'chck.biol.isfeaturesfile': ('biol.html#isfeaturesfile', 'chck/biol.py'),
                           'chck.biol.isfeaturespath': ('biol.html#isfeaturespath', 'chck/biol.py'),
                           'chck.biol.isfilteredmatrixdir': ('biol.html#isfilteredmatrixdir', 'chck/biol.py'),
                           'chck.biol.isfilteredmatrixpath': ('biol.html#isfilteredmatrixpath', 'chck/biol.py'),
                           'chck.biol.isfmat': ('biol.html#isfmat', 'chck/biol.py'),
                           'chck.biol.isfmatdir': ('biol.html#isfmatdir', 'chck/biol.py'),
                           'chck.biol.ismatrixfile': ('biol.html#ismatrixfile', 'chck/biol.py'),
                           'chck.biol.ismatrixpath': ('biol.html#ismatrixpath', 'chck/biol.py'),
                           'chck.biol.issens': ('biol.html#issens', 'chck/biol.py'),
                           'chck.biol.issensdir': ('biol.html#issensdir', 'chck/biol.py'),
                           'chck.biol.issensitivitydir': ('biol.html#issensitivitydir', 'chck/biol.py'),
                           'chck.biol.issensitivitypath': ('biol.html#issensitivitypath', 'chck/biol.py')},
            'chck.bool': { 'chck.bool.is01': ('bool.html#is01', 'chck/bool.py'),
                           'chck.bool.isbcall': ('bool.html#isbcall', 'chck/bool.py'),
                           'chck.bool.isboolcall': ('bool.html#isboolcall', 'chck/bool.py'),
                           'chck.bool.isbooltype': ('bool.html#isbooltype', 'chck/bool.py'),
                           'chck.bool.isbooly': ('bool.html#isbooly', 'chck/bool.py'),
                           'chck.bool.isbtype': ('bool.html#isbtype', 'chck/bool.py')},
            'chck.call': { 'chck.call.hasget': ('call.html#hasget', 'chck/call.py'),
                           'chck.call.haskeys': ('call.html#haskeys', 'chck/call.py'),
                           'chck.call.isattrcall': ('call.html#isattrcall', 'chck/call.py'),
                           'chck.call.iscallattr': ('call.html#iscallattr', 'chck/call.py')},
            'chck.comp': {'chck.comp.islistordict': ('comp.html#islistordict', 'chck/comp.py')},
            'chck.core': { 'chck.core.isany': ('core.html#isany', 'chck/core.py'),
                           'chck.core.isbool': ('core.html#isbool', 'chck/core.py'),
                           'chck.core.isbytes': ('core.html#isbytes', 'chck/core.py'),
                           'chck.core.iscall': ('core.html#iscall', 'chck/core.py'),
                           'chck.core.isclass': ('core.html#isclass', 'chck/core.py'),
                           'chck.core.iscls': ('core.html#iscls', 'chck/core.py'),
                           'chck.core.isdict': ('core.html#isdict', 'chck/core.py'),
                           'chck.core.isint': ('core.html#isint', 'chck/core.py'),
                           'chck.core.isiter': ('core.html#isiter', 'chck/core.py'),
                           'chck.core.islist': ('core.html#islist', 'chck/core.py'),
                           'chck.core.isnone': ('core.html#isnone', 'chck/core.py'),
                           'chck.core.isnum': ('core.html#isnum', 'chck/core.py'),
                           'chck.core.isrange': ('core.html#isrange', 'chck/core.py'),
                           'chck.core.isrng': ('core.html#isrng', 'chck/core.py'),
                           'chck.core.isset': ('core.html#isset', 'chck/core.py'),
                           'chck.core.isslc': ('core.html#isslc', 'chck/core.py'),
                           'chck.core.isslice': ('core.html#isslice', 'chck/core.py'),
                           'chck.core.isstr': ('core.html#isstr', 'chck/core.py'),
                           'chck.core.istup': ('core.html#istup', 'chck/core.py'),
                           'chck.core.istuple': ('core.html#istuple', 'chck/core.py'),
                           'chck.core.istype': ('core.html#istype', 'chck/core.py'),
                           'chck.core.notnone': ('core.html#notnone', 'chck/core.py')},
            'chck.dcls': {'chck.dcls.iskwotype': ('dcls.html#iskwotype', 'chck/dcls.py')},
            'chck.dict': { 'chck.dict.isdictitems': ('dict.html#isdictitems', 'chck/dict.py'),
                           'chck.dict.isdictiter': ('dict.html#isdictiter', 'chck/dict.py'),
                           'chck.dict.isdictkeys': ('dict.html#isdictkeys', 'chck/dict.py'),
                           'chck.dict.isdicttype': ('dict.html#isdicttype', 'chck/dict.py'),
                           'chck.dict.isdictvalues': ('dict.html#isdictvalues', 'chck/dict.py'),
                           'chck.dict.isditems': ('dict.html#isditems', 'chck/dict.py'),
                           'chck.dict.isditer': ('dict.html#isditer', 'chck/dict.py'),
                           'chck.dict.isdkeys': ('dict.html#isdkeys', 'chck/dict.py'),
                           'chck.dict.isdvals': ('dict.html#isdvals', 'chck/dict.py')},
            'chck.fref': { 'chck.fref.dereference': ('fref.html#dereference', 'chck/fref.py'),
                           'chck.fref.isforwardref': ('fref.html#isforwardref', 'chck/fref.py'),
                           'chck.fref.isfref': ('fref.html#isfref', 'chck/fref.py')},
            'chck.insp': { 'chck.insp.isclsarg': ('insp.html#isclsarg', 'chck/insp.py'),
                           'chck.insp.isempty': ('insp.html#isempty', 'chck/insp.py'),
                           'chck.insp.isfunc': ('insp.html#isfunc', 'chck/insp.py'),
                           'chck.insp.iskwonly': ('insp.html#iskwonly', 'chck/insp.py'),
                           'chck.insp.islambda': ('insp.html#islambda', 'chck/insp.py'),
                           'chck.insp.isp': ('insp.html#isp', 'chck/insp.py'),
                           'chck.insp.ispcls': ('insp.html#ispcls', 'chck/insp.py'),
                           'chck.insp.ispempty': ('insp.html#ispempty', 'chck/insp.py'),
                           'chck.insp.ispkwonly': ('insp.html#ispkwonly', 'chck/insp.py'),
                           'chck.insp.ispself': ('insp.html#ispself', 'chck/insp.py'),
                           'chck.insp.isself': ('insp.html#isself', 'chck/insp.py'),
                           'chck.insp.issig': ('insp.html#issig', 'chck/insp.py'),
                           'chck.insp.isspec': ('insp.html#isspec', 'chck/insp.py'),
                           'chck.insp.isvarkwd': ('insp.html#isvarkwd', 'chck/insp.py'),
                           'chck.insp.isvarpos': ('insp.html#isvarpos', 'chck/insp.py')},
            'chck.isad': { 'chck.isad.isad': ('isad.html#isad', 'chck/isad.py'),
                           'chck.isad.isanndata': ('isad.html#isanndata', 'chck/isad.py')},
            'chck.isan': { 'chck.isan.isad': ('isan.html#isad', 'chck/isan.py'),
                           'chck.isan.isanndata': ('isan.html#isanndata', 'chck/isan.py')},
            'chck.iscv': { 'chck.iscv.isvid': ('iscv.html#isvid', 'chck/iscv.py'),
                           'chck.iscv.isvideo': ('iscv.html#isvideo', 'chck/iscv.py'),
                           'chck.iscv.isvideocapture': ('iscv.html#isvideocapture', 'chck/iscv.py')},
            'chck.isnp': { 'chck.isnp.isarr': ('isnp.html#isarr', 'chck/isnp.py'),
                           'chck.isnp.ismtx': ('isnp.html#ismtx', 'chck/isnp.py'),
                           'chck.isnp.isnparr': ('isnp.html#isnparr', 'chck/isnp.py'),
                           'chck.isnp.isnparrary': ('isnp.html#isnparrary', 'chck/isnp.py'),
                           'chck.isnp.isnpmatrix': ('isnp.html#isnpmatrix', 'chck/isnp.py'),
                           'chck.isnp.isnpmtx': ('isnp.html#isnpmtx', 'chck/isnp.py'),
                           'chck.isnp.isnpnan': ('isnp.html#isnpnan', 'chck/isnp.py')},
            'chck.ispd': { 'chck.ispd.isbidx': ('ispd.html#isbidx', 'chck/ispd.py'),
                           'chck.ispd.isboolidx': ('ispd.html#isboolidx', 'chck/ispd.py'),
                           'chck.ispd.isboolindex': ('ispd.html#isboolindex', 'chck/ispd.py'),
                           'chck.ispd.iscategoricalindex': ('ispd.html#iscategoricalindex', 'chck/ispd.py'),
                           'chck.ispd.iscatidx': ('ispd.html#iscatidx', 'chck/ispd.py'),
                           'chck.ispd.iscidx': ('ispd.html#iscidx', 'chck/ispd.py'),
                           'chck.ispd.isdf': ('ispd.html#isdf', 'chck/ispd.py'),
                           'chck.ispd.isidx': ('ispd.html#isidx', 'chck/ispd.py'),
                           'chck.ispd.isidxlike': ('ispd.html#isidxlike', 'chck/ispd.py'),
                           'chck.ispd.isindexlike': ('ispd.html#isindexlike', 'chck/ispd.py'),
                           'chck.ispd.ispdna': ('ispd.html#ispdna', 'chck/ispd.py'),
                           'chck.ispd.isseries': ('ispd.html#isseries', 'chck/ispd.py'),
                           'chck.ispd.notpdna': ('ispd.html#notpdna', 'chck/ispd.py')},
            'chck.issp': { 'chck.issp.issparr': ('issp.html#issparr', 'chck/issp.py'),
                           'chck.issp.issparray': ('issp.html#issparray', 'chck/issp.py'),
                           'chck.issp.issparse': ('issp.html#issparse', 'chck/issp.py'),
                           'chck.issp.isspmatrix': ('issp.html#isspmatrix', 'chck/issp.py'),
                           'chck.issp.isspmtx': ('issp.html#isspmtx', 'chck/issp.py'),
                           'chck.issp.isspr': ('issp.html#isspr', 'chck/issp.py')},
            'chck.iter': { 'chck.iter.isiprod': ('iter.html#isiprod', 'chck/iter.py'),
                           'chck.iter.isiproduct': ('iter.html#isiproduct', 'chck/iter.py'),
                           'chck.iter.isiternotstr': ('iter.html#isiternotstr', 'chck/iter.py'),
                           'chck.iter.isiternotstrbytes': ('iter.html#isiternotstrbytes', 'chck/iter.py'),
                           'chck.iter.isiterprod': ('iter.html#isiterprod', 'chck/iter.py'),
                           'chck.iter.isiterproduct': ('iter.html#isiterproduct', 'chck/iter.py'),
                           'chck.iter.isproduct': ('iter.html#isproduct', 'chck/iter.py'),
                           'chck.iter.iternotstr': ('iter.html#iternotstr', 'chck/iter.py'),
                           'chck.iter.notiterstr': ('iter.html#notiterstr', 'chck/iter.py')},
            'chck.itrs': { 'chck.itrs.alldig': ('itrs.html#alldig', 'chck/itrs.py'),
                           'chck.itrs.allint': ('itrs.html#allint', 'chck/itrs.py'),
                           'chck.itrs.allmap': ('itrs.html#allmap', 'chck/itrs.py'),
                           'chck.itrs.alltype': ('itrs.html#alltype', 'chck/itrs.py')},
            'chck.lens': { 'chck.lens.has1': ('lens.html#has1', 'chck/lens.py'),
                           'chck.lens.has2': ('lens.html#has2', 'chck/lens.py'),
                           'chck.lens.has3': ('lens.html#has3', 'chck/lens.py'),
                           'chck.lens.has4': ('lens.html#has4', 'chck/lens.py'),
                           'chck.lens.hasn': ('lens.html#hasn', 'chck/lens.py'),
                           'chck.lens.is2ints': ('lens.html#is2ints', 'chck/lens.py'),
                           'chck.lens.is3ints': ('lens.html#is3ints', 'chck/lens.py'),
                           'chck.lens.is4ints': ('lens.html#is4ints', 'chck/lens.py'),
                           'chck.lens.ischar': ('lens.html#ischar', 'chck/lens.py'),
                           'chck.lens.ischr': ('lens.html#ischr', 'chck/lens.py'),
                           'chck.lens.islen1': ('lens.html#islen1', 'chck/lens.py'),
                           'chck.lens.islen2': ('lens.html#islen2', 'chck/lens.py'),
                           'chck.lens.islen3': ('lens.html#islen3', 'chck/lens.py'),
                           'chck.lens.islen4': ('lens.html#islen4', 'chck/lens.py'),
                           'chck.lens.islenn': ('lens.html#islenn', 'chck/lens.py'),
                           'chck.lens.isnints': ('lens.html#isnints', 'chck/lens.py')},
            'chck.misc': { 'chck.misc.arein': ('misc.html#arein', 'chck/misc.py'),
                           'chck.misc.isin': ('misc.html#isin', 'chck/misc.py'),
                           'chck.misc.isna': ('misc.html#isna', 'chck/misc.py'),
                           'chck.misc.notna': ('misc.html#notna', 'chck/misc.py')},
            'chck.path': { 'chck.path.isdir': ('path.html#isdir', 'chck/path.py'),
                           'chck.path.isfile': ('path.html#isfile', 'chck/path.py'),
                           'chck.path.ishidden': ('path.html#ishidden', 'chck/path.py'),
                           'chck.path.ispath': ('path.html#ispath', 'chck/path.py'),
                           'chck.path.ispathlike': ('path.html#ispathlike', 'chck/path.py'),
                           'chck.path.ispathy': ('path.html#ispathy', 'chck/path.py'),
                           'chck.path.isxath': ('path.html#isxath', 'chck/path.py')},
            'chck.pyty': { 'chck.pyty.isbuiltin': ('pyty.html#isbuiltin', 'chck/pyty.py'),
                           'chck.pyty.isbuiltinfunctiontype': ('pyty.html#isbuiltinfunctiontype', 'chck/pyty.py'),
                           'chck.pyty.ismethodwrapper': ('pyty.html#ismethodwrapper', 'chck/pyty.py'),
                           'chck.pyty.ismthdwrap': ('pyty.html#ismthdwrap', 'chck/pyty.py')},
            'chck.strs': { 'chck.strs.isallstr': ('strs.html#isallstr', 'chck/strs.py'),
                           'chck.strs.ischar': ('strs.html#ischar', 'chck/strs.py'),
                           'chck.strs.ischars': ('strs.html#ischars', 'chck/strs.py'),
                           'chck.strs.isdig': ('strs.html#isdig', 'chck/strs.py'),
                           'chck.strs.isdotstr': ('strs.html#isdotstr', 'chck/strs.py'),
                           'chck.strs.isdunder': ('strs.html#isdunder', 'chck/strs.py'),
                           'chck.strs.iselps': ('strs.html#iselps', 'chck/strs.py'),
                           'chck.strs.islower': ('strs.html#islower', 'chck/strs.py'),
                           'chck.strs.ismangled': ('strs.html#ismangled', 'chck/strs.py'),
                           'chck.strs.isnilstr': ('strs.html#isnilstr', 'chck/strs.py'),
                           'chck.strs.ispipe': ('strs.html#ispipe', 'chck/strs.py'),
                           'chck.strs.ispipestr': ('strs.html#ispipestr', 'chck/strs.py'),
                           'chck.strs.isprivate': ('strs.html#isprivate', 'chck/strs.py'),
                           'chck.strs.ispublic': ('strs.html#ispublic', 'chck/strs.py'),
                           'chck.strs.ispunc': ('strs.html#ispunc', 'chck/strs.py'),
                           'chck.strs.isquote': ('strs.html#isquote', 'chck/strs.py'),
                           'chck.strs.isstar': ('strs.html#isstar', 'chck/strs.py'),
                           'chck.strs.isstrbytes': ('strs.html#isstrbytes', 'chck/strs.py'),
                           'chck.strs.issunder': ('strs.html#issunder', 'chck/strs.py'),
                           'chck.strs.isuniqchrs': ('strs.html#isuniqchrs', 'chck/strs.py'),
                           'chck.strs.isupper': ('strs.html#isupper', 'chck/strs.py'),
                           'chck.strs.notstrbytes': ('strs.html#notstrbytes', 'chck/strs.py')},
            'chck.tgrd': { 'chck.tgrd.isinst': ('tgrd.html#isinst', 'chck/tgrd.py'),
                           'chck.tgrd.isinstq': ('tgrd.html#isinstq', 'chck/tgrd.py'),
                           'chck.tgrd.isof': ('tgrd.html#isof', 'chck/tgrd.py'),
                           'chck.tgrd.istypeq': ('tgrd.html#istypeq', 'chck/tgrd.py')},
            'chck.trch': { 'chck.trch.iscpu': ('trch.html#iscpu', 'chck/trch.py'),
                           'chck.trch.iscputensor': ('trch.html#iscputensor', 'chck/trch.py'),
                           'chck.trch.iscuda': ('trch.html#iscuda', 'chck/trch.py'),
                           'chck.trch.isdataframedataset': ('trch.html#isdataframedataset', 'chck/trch.py'),
                           'chck.trch.isdataloader': ('trch.html#isdataloader', 'chck/trch.py'),
                           'chck.trch.isdatamodule': ('trch.html#isdatamodule', 'chck/trch.py'),
                           'chck.trch.isdataset': ('trch.html#isdataset', 'chck/trch.py'),
                           'chck.trch.isdevice': ('trch.html#isdevice', 'chck/trch.py'),
                           'chck.trch.isdfdataset': ('trch.html#isdfdataset', 'chck/trch.py'),
                           'chck.trch.isdfds': ('trch.html#isdfds', 'chck/trch.py'),
                           'chck.trch.isdl': ('trch.html#isdl', 'chck/trch.py'),
                           'chck.trch.isdm': ('trch.html#isdm', 'chck/trch.py'),
                           'chck.trch.isds': ('trch.html#isds', 'chck/trch.py'),
                           'chck.trch.isdtype': ('trch.html#isdtype', 'chck/trch.py'),
                           'chck.trch.ismps': ('trch.html#ismps', 'chck/trch.py'),
                           'chck.trch.ismpstensor': ('trch.html#ismpstensor', 'chck/trch.py'),
                           'chck.trch.istens': ('trch.html#istens', 'chck/trch.py'),
                           'chck.trch.istensor': ('trch.html#istensor', 'chck/trch.py'),
                           'chck.trch.istorchtype': ('trch.html#istorchtype', 'chck/trch.py'),
                           'chck.trch.istrc': ('trch.html#istrc', 'chck/trch.py'),
                           'chck.trch.istrctype': ('trch.html#istrctype', 'chck/trch.py'),
                           'chck.trch.isttype': ('trch.html#isttype', 'chck/trch.py')},
            'chck.wrap': { 'chck.wrap.decoguard': ('wrap.html#decoguard', 'chck/wrap.py'),
                           'chck.wrap.guardanns': ('wrap.html#guardanns', 'chck/wrap.py'),
                           'chck.wrap.guarddocs': ('wrap.html#guarddocs', 'chck/wrap.py'),
                           'chck.wrap.guardname': ('wrap.html#guardname', 'chck/wrap.py'),
                           'chck.wrap.makeguard': ('wrap.html#makeguard', 'chck/wrap.py'),
                           'chck.wrap.setwrapattrs': ('wrap.html#setwrapattrs', 'chck/wrap.py'),
                           'chck.wrap.wrapguard': ('wrap.html#wrapguard', 'chck/wrap.py')}}}



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/12_iter.ipynb.

# %% auto 0
__all__ = ['isiterproduct', 'isiproduct', 'isiterprod', 'isproduct', 'isiprod', 'isiternotstrbytes', 'isiternotstr', 'notiterstr',
           'iternotstr']

# %% ../nbs/12_iter.ipynb 4
import itertools
from typing import TypeGuard

from typing import Iterable

# %% ../nbs/12_iter.ipynb 5
from .atyp import sbytes, notsbytes, StrBytes, NotStrBytes
from .core import isiter
from .strs import notstrbytes

# %% ../nbs/12_iter.ipynb 7
def isiterproduct(x) -> TypeGuard[itertools.product]:
    '''Checks if `x` is an `itertools.product`.
    
    See Also
    --------
    isiprod : Alias for `isiterproduct`.
    isproduct : Alias for `isiterproduct`.
    isiproduct : Alias for `isiterproduct`.
    isiterprod : Alias for `isiterproduct`.
    '''
    return isiter(x) and isinstance(x, itertools.product)

def isiproduct(x) -> TypeGuard[itertools.product]:
    '''Checks if `x` is an `itertools.product`.
    
    See Also
    --------
    isiprod : Alias for `isiproduct`.
    isproduct : Alias for `isiproduct`.    
    isiterprod : Alias for `isiproduct`.
    isiterproduct : Alias for `isiproduct`.
    '''
    return isiterproduct(x)

def isiterprod(x) -> TypeGuard[itertools.product]:
    '''Checks if `x` is an `itertools.product`.
    
    See Also
    --------
    isiprod : Alias for `isiterprod`.
    isproduct : Alias for `isiterprod`.
    isiproduct : Alias for `isiterprod`.
    isiterproduct : Alias for `isiterprod`.
    '''
    return isiterproduct(x)

def isproduct(x) -> TypeGuard[itertools.product]:
    '''Checks if `x` is an `itertools.product`.
    
    See Also
    --------
    isiprod : Alias for `isproduct`.
    isiproduct : Alias for `isproduct`.
    isiterprod : Alias for `isproduct`.
    isiterproduct : Alias for `isproduct`.
    '''
    return isiterproduct(x)

def isiprod(x) -> TypeGuard[itertools.product]:
    '''Checks if `x` is an `itertools.product`.
    
    See Also
    --------
    isproduct : Alias for `isiprod`.
    isiproduct : Alias for `isiprod`.
    isiterprod : Alias for `isiprod`.
    isiterproduct : Alias for `isiprod`.
    '''
    return isiterproduct(x)

# %% ../nbs/12_iter.ipynb 8
def isiternotstrbytes(x) -> TypeGuard[Iterable[notsbytes]]:
    '''Checks if `x` is an iterable containing elements other than strings or bytes.
    
    See Also
    --------
    isiternotstr : Alias for `isiternotstrbytes`.
    notiterstr : Alias for `isiternotstrbytes`.
    iternotstr : Alias for `isiternotstrbytes`.
    '''
    return isiter(x) and notstrbytes(x)


def isiternotstr(x) -> TypeGuard[Iterable[notsbytes]]:
    '''Checks if `x` is an iterable containing elements other than strings or bytes.
    
    See Also
    --------
    isiternotstrbytes : Alias for `isiternotstr`.
    notiterstr : Alias for `isiternotstr`.
    iternotstr : Alias for `isiternotstr`.
    '''
    return isiternotstrbytes(x)


def notiterstr(x) -> TypeGuard[Iterable[notsbytes]]:
    '''Checks if `x` is an iterable containing elements other than strings or bytes.
    
    See Also
    --------
    isiternotstrbytes : Alias for `notiterstr`.
    isiternotstr : Alias for `notiterstr`.
    iternotstr : Alias for `notiterstr`.
    '''
    return isiternotstrbytes(x)


def iternotstr(x) -> TypeGuard[Iterable[notsbytes]]:
    '''Checks if `x` is an iterable containing elements other than strings or bytes.
    
    See Also
    --------
    isiternotstrbytes : Alias for `iternotstr`.
    isiternotstr : Alias for `iternotstr`.
    notiterstr : Alias for `iternotstr`.
    '''
    return isiternotstrbytes(x)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/30_insp.ipynb.

# %% auto 0
__all__ = ['issig', 'isspec', 'isp', 'isvarpos', 'isvarkwd', 'isself', 'isclsarg', 'ispself', 'ispcls', 'isempty', 'ispempty',
           'iskwonly', 'ispkwonly', 'isfunc', 'islambda']

# %% ../nbs/30_insp.ipynb 4
from inspect import Parameter, Signature, isfunction, FullArgSpec
from types import FunctionType, LambdaType, BuiltinFunctionType, BuiltinMethodType
from typing import TypeGuard

from typing import Literal

# %% ../nbs/30_insp.ipynb 6
def issig(x) -> TypeGuard[Signature]:
    '''Check if x is of type `Signature`.'''
    return isinstance(x, Signature)

def isspec(x) -> TypeGuard[FullArgSpec]:
    '''Check if x is of type `FullArgSpec`.'''
    return isinstance(x, FullArgSpec)

# %% ../nbs/30_insp.ipynb 8
def isp(p: Parameter) -> TypeGuard[Parameter]:
    '''Check if parameter is of type `Parameter`.'''
    return isinstance(p, Parameter)

# %% ../nbs/30_insp.ipynb 9
def isvarpos(p: Parameter) -> TypeGuard[Parameter.VAR_POSITIONAL]:
    '''Check if parameter is of type `Parameter.VAR_POSITIONAL`.'''
    return isp(p) and p.kind == Parameter.VAR_POSITIONAL

def isvarkwd(p: Parameter) -> TypeGuard[Parameter.VAR_KEYWORD]:
    '''Check if parameter is of type `Parameter.VAR_KEYWORD`.'''
    return isp(p) and p.kind == Parameter.VAR_KEYWORD

# %% ../nbs/30_insp.ipynb 11
# NOTE: not sure if this is the same as `isinstance(x, Self)`
def isself(x) -> TypeGuard[Literal['self']]:
    '''Check if `x` is `"self"`.'''
    return x == 'self' 

# %% ../nbs/30_insp.ipynb 13
def isclsarg(x) -> TypeGuard[Literal['cls']]:
    '''Check if `x` is `"cls"`.'''
    return x == 'cls' 

# %% ../nbs/30_insp.ipynb 15
def ispself(p: Parameter) -> TypeGuard[Parameter]:
    '''Check if `p: Parameter` has `name = "self"`.'''
    return isp(p) and isself(p.name)

def ispcls(p: Parameter) -> TypeGuard[Parameter]:
    '''Check if `p: Parameter` has `name = "cls"`.'''
    return isp(p) and p.name == 'cls'

# %% ../nbs/30_insp.ipynb 17
def isempty(x) -> TypeGuard[Parameter.empty]:
    '''Check if `x` is `Parameter.empty`.'''
    return x == Parameter.empty

def ispempty(p: Parameter) -> TypeGuard[Parameter.empty]:
    '''Check if `p: Parameter` is of type `Parameter.empty`.'''
    return isp(p) and isempty(p.default)

# %% ../nbs/30_insp.ipynb 19
def iskwonly(x) -> TypeGuard[Parameter.KEYWORD_ONLY]:
    '''Check if `x` is `Parameter.KEYWORD_ONLY`.'''
    return x == Parameter.KEYWORD_ONLY

def ispkwonly(p: Parameter) -> TypeGuard[Parameter.KEYWORD_ONLY]:
    '''Check if `p: Parameter` is of type `Parameter.KEYWORD_ONLY`.'''
    return isp(p) and iskwonly(p.kind)

# %% ../nbs/30_insp.ipynb 21
def isfunc(x) -> TypeGuard[FunctionType]:
    '''Check if `x` is a function.'''
    return isfunction(x)

def islambda(f) -> TypeGuard[LambdaType]:
    '''Check if `x` is a lambda function.'''
    return isfunction(f) and f.__name__ == '<lambda>'



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/100_itrs.ipynb.

# %% auto 0
__all__ = ['allmap', 'alltype', 'allint', 'alldig']

# %% ../nbs/100_itrs.ipynb 4
import inspect
from numbers import Number
from typing import Any, Type, Callable, Iterable, TypeGuard

# %% ../nbs/100_itrs.ipynb 5
from .core import isiter, isint, isstr
from .strs import isdig
from .wrap import vanilla, isguard

# %% ../nbs/100_itrs.ipynb 7
def allmap(x, f: Callable[[Any], TypeGuard[Type]]) -> bool:
    return isiter(x) and all(map(f, x))

def alltype(x, t: type) -> bool:
    return isiter(x) and all([vanilla(t)(y) for y in x])

def allint(x): return allmap(x, isint)

def alldig(x): return isiter(x) and allmap(x, isdig)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/60_isnp.ipynb.

# %% auto 0
__all__ = ['isnpnan', 'isnpmatrix', 'isnparrary', 'isnpmtx', 'isnparr', 'ismtx', 'isarr']

# %% ../nbs/60_isnp.ipynb 4
from typing import TypeGuard
from typing import Union

# %% ../nbs/60_isnp.ipynb 5
from .atyp import (npnan, npNaN, npNAN, npmatrix, npndarray)
from .atyp import (nan, NaN, NAN, matrix, ndarray)

# %% ../nbs/60_isnp.ipynb 7
def isnpnan(x) -> TypeGuard[Union[npnan, npNaN, npNAN]]:
    "Check if `x` is any of  `np.nan`, `np.NaN`, or `np.NAN`."
    return x is nan or x is NaN or x is NAN

# %% ../nbs/60_isnp.ipynb 8
def isnpmatrix(x) -> TypeGuard[npmatrix]:
    '''Check if `x` is an `np.matrix`.
    
    See Also
    --------
    isnpmtx : Alias for `isnpmatrix`.
    '''
    return isinstance(x, matrix)

def isnparrary(x) -> TypeGuard[npndarray]:
    '''Check if `x` is an `np.ndarray` (and not an `np.matrix`).
    
    Notes
    -----
    `np.matrix` is a subclass of `np.ndarray`, so `isinstance(x, np.ndarray)` 
    will return `True` for `np.matrix` objects.

    See Also
    --------
    isnparr : Alias for `isnparrary`.
    '''
    return isinstance(x, ndarray) and not isnpmatrix(x)


def isnpmtx(x) -> TypeGuard[npmatrix]:
    '''Check if `x` is an `np.matrix`.

    See Also
    --------
    isnpmatrix : Alias for `isnpmtx`.
    '''
    return isnpmatrix(x)

def isnparr(x) -> TypeGuard[npndarray]:
    '''Check if `x` is an `np.ndarray` (and not an `np.matrix`).
    
    Notes
    -----
    `np.matrix` is a subclass of `np.ndarray`, so `isinstance(x, np.ndarray)` 
    will return `True` for `np.matrix` objects.
    
    See Also
    --------
    isnparrary : Alias for `isnparr`.
    '''
    return isnparrary(x)


def ismtx(x) -> TypeGuard[npmatrix]:
    '''Check if `x` is an `np.matrix`.
    
    See Also
    --------
    isnpmatrix : Check if `x` is an `np.matrix`.
    isnpmtx : Alias for `isnpmatrix`.
    '''    
    return isnpmatrix(x)

def isarr(x) -> TypeGuard[npndarray]:
    '''Check if `x` is an `np.ndarray` (and not an `np.matrix`).
    
    Notes
    -----
    `np.matrix` is a subclass of `np.ndarray`, so `isinstance(x, np.ndarray)` 
    will return `True` for `np.matrix` objects.
    
    See Also
    --------
    isnparrary : Check if `x` is an `np.ndarray`.
    isnparr : Alias for `isnparrary`.
    '''
    return isnparrary(x)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/90_alls.ipynb.

# %% auto 0
__all__ = ['allmap', 'areinstances', 'alltype', 'allsame', 'allint', 'alldig', 'allstr', 'isints', 'isstrs']

# %% ../nbs/90_alls.ipynb 4
from typing import Iterable, TypeGuard

# %% ../nbs/90_alls.ipynb 5
from .atyp import G, GuardFunc
from .core import isiter, isint, isstr
from .wrap import makeguard
from .strs import isdig

# %% ../nbs/90_alls.ipynb 7
def allmap(x, f: GuardFunc) -> bool:
    '''Maps `TypeGuard` function `f` over `x` and returns `all` of the results'''
    return isiter(x) and all(map(f, x))

def areinstances(x, t: G) -> TypeGuard[Iterable[G]]:
    '''Checks if all elements of `x` are instances of `t`
    
    Notes
    -----
    The following are all functionally equivalent to the current implementation:
    ```python
    allmap(x, vanilla(t))
    allmap(x, lambda y: isinstance(y, t))
    isiter(x) and all(isinstance(y, t) for y in x)
    isiter(x) and all(vanilla(t)(y) for y in x)
    isiter(x) and all(map(lambda y: isinstance(y, t), x))
    
    # NOTE: dtype: TypeLike, isfunc: GuardFunc = isinstance
    isiter(vals) and all(isfunc(val, dtype) for val in vals) 
    ```

    See Also
    --------
    allmap : Maps `f` over `x` and returns `all` of the results
    alltype : Alias for `areinstances`
    '''
    return allmap(x, makeguard(t))

# %% ../nbs/90_alls.ipynb 9
def alltype(x, t: G) -> TypeGuard[Iterable[G]]: 
    '''Checks if all elements of `x` are of type `t`'''
    return areinstances(x, t)

def allsame(x: Iterable) -> TypeGuard[Iterable[G]]:
    '''Checks if all elements of `x` are of the same type'''
    return alltype(x, type(x[0])) if (isiter(x) and len(x) > 0) else True

# %% ../nbs/90_alls.ipynb 11
def allint(x) -> TypeGuard[Iterable[int]]: 
    '''Check if all elements of `x` are an integer
    
    See Also
    --------
    isints : Alias for `allint`
    '''
    return allmap(x, isint)

def alldig(x) -> TypeGuard[Iterable[str]]: 
    '''Check if all elements of `x` are a digit string'''
    return allmap(x, isdig)

def allstr(x) -> TypeGuard[Iterable[str]]: 
    '''Check if all elements of `x` are a string
    
    See Also
    --------
    isstrs : Alias for `allstr`
    '''
    return allmap(x, isstr)

# %% ../nbs/90_alls.ipynb 13
def isints(x) -> TypeGuard[Iterable[int]]:
    '''Check if all elements of `x` are an integer.
     
    See Also
    --------
    allint : Alias for `isints`
    '''
    return allint(x)


def isstrs(x) -> TypeGuard[Iterable[str]]:
    '''Check if all elements of `x` are a string.
     
    See Also
    --------
    allstr : Alias for `isstrs`
    '''
    return allstr(x)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/31_dcls.ipynb.

# %% auto 0
__all__ = ['iskwotype']

# %% ../nbs/31_dcls.ipynb 4
from typing import TypeGuard
from dataclasses import _KW_ONLY_TYPE

# %% ../nbs/31_dcls.ipynb 6
def iskwotype(x) -> TypeGuard[_KW_ONLY_TYPE]:
    '''Check if `x` is of type `dataclasses.KW_ONLY`.'''
    return isinstance(x, _KW_ONLY_TYPE)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/63_trch.ipynb.

# %% auto 0
__all__ = ['isdevice', 'istensor', 'istrc', 'istens', 'istorchtype', 'isdtype', 'isttype', 'istrctype', 'iscpu', 'ismps',
           'iscuda', 'iscputensor', 'ismpstensor', 'isdataset', 'isds', 'isdataloader', 'isdl', 'isdatamodule', 'isdm',
           'isdfds', 'isdfdataset', 'isdataframedataset']

# %% ../nbs/63_trch.ipynb 4
from typing import TypeGuard
from typing import Literal

# %% ../nbs/63_trch.ipynb 5
from .atyp import (dtype, device, Tensor, pttype, ptdevice, pttensor)
from .atyp import (Dataset, DataLoader, ptdataset, ptdataloader)
from .atyp import (pldatamodule, DataModule)

from .core import isstr, isnone

# %% ../nbs/63_trch.ipynb 6
from .ispd import isdf

# %% ../nbs/63_trch.ipynb 9
def isdevice(x) -> TypeGuard[ptdevice]:
    '''Check if `x` is a `torch.device`'''
    return not isnone(x) and ((not isstr(x) and hasattr(x, 'type')) or isinstance(x, device))

def istensor(x) -> TypeGuard[pttensor]:
    '''Check if `x` is a `torch.Tensor`
    
    See Also
    --------
    istrc : Alias for `istensor`
    istens : Alias for `istensor`
    '''
    return not isnone(x) and isinstance(x, Tensor) and hasattr(x, 'device')

def istrc(x) -> TypeGuard[ptdevice]:
    '''Check if `x` is a `torch.Tensor` with a `torch.device`.
    
    See Also
    --------
    istensor : Alias for `istrc`
    '''
    return istensor(x)

def istens(x) -> TypeGuard[pttensor]:
    '''Check if `x` is a `torch.Tensor` with a `torch.device`.
    
    See Also
    --------
    istensor : Check if `x` is a `torch.Tensor` with a `torch.device`
    istrc : Alias for `istensor`
    '''
    return istensor(x)



# %% ../nbs/63_trch.ipynb 10
def istorchtype(x) -> TypeGuard[pttype]:
    '''Check if `x` is subclass of `torch.dtype`
    
     See Also
    --------
    isdtype : Alias for `istorchtype`
    isttype : Alias for `istorchtype`
    istrctype : Alias for `istorchtype`
    '''
    return not isnone(x) and isinstance(x, dtype)

def isdtype(x) -> TypeGuard[pttype]:
    '''Check if `x` is subclass of `torch.dtype`
    
     See Also
    --------
    isttype : Alias for `isdtype`
    istrctype : Alias for `isdtype`
    istorchtype : Alias for `isdtype`
    '''
    return istorchtype(x)


def isttype(x) -> TypeGuard[pttype]:
    '''Check if `x` is subclass of `torch.dtype`
    
     See Also
    --------
    isdtype : Alias for `isttype`
    istrctype : Alias for `isttype`
    istorchtype : Alias for `isttype`
    '''
    return istorchtype(x)

def istrctype(x) -> TypeGuard[pttype]:
    '''Check if `x` is subclass of `torch.dtype`
    
     See Also
    --------
    isdtype : Alias for `istrctype`
    isttype : Alias for `istrctype`
    istorchtype : Alias for `istrctype`
    '''
    return istorchtype(x)


# %% ../nbs/63_trch.ipynb 12
def iscpu(s: str) -> TypeGuard[Literal['cpu']]:
    '''Check if `s` is `'cpu'`'''
    return s == 'cpu'

def ismps(s: str) -> TypeGuard[Literal['mps']]:
    '''Check if `s` is `'mps'`'''
    return s == 'mps'

def iscuda(s: str) -> TypeGuard[Literal['cuda']]:
    '''Check if `s` is `'cuda'`'''
    return s == 'cuda'

# %% ../nbs/63_trch.ipynb 14
def iscputensor(x) -> TypeGuard[pttensor]:
    '''Check if `x` is a `torch.Tensor` on cpu'''
    return istensor(x) and iscpu(x.device.type)

def ismpstensor(x) -> TypeGuard[pttensor]:
    '''Check if `x` is a `torch.Tensor` on mps'''
    return istensor(x) and ismps(x.device.type)

def ismpstensor(x) -> TypeGuard[pttensor]:
    '''Check if `x` is a `torch.Tensor` on cuda'''
    return istensor(x) and iscuda(x.device.type)

# %% ../nbs/63_trch.ipynb 16
def isdataset(x) -> TypeGuard[ptdataset]:
    '''Check if `x` is a `torch.utils.data.Dataset`
    
    See Also
    --------
    isds : Alias for `isdataset`    
    '''
    return not isnone(x) and isinstance(x, Dataset)

def isds(x) -> TypeGuard[ptdataset]:
    '''Check if `x` is a `torch.utils.data.Dataset`
    
    See Also
    --------
    isdataset : Alias for `isds`
    '''
    return isdataset(x)

def isdataloader(x) -> TypeGuard[ptdataloader]:
    '''Check if `x` is a `torch.utils.data.DataLoader`
    
    See Also
    --------
    isds : Alias for `isdataloader`    
    '''
    return not isnone(x) and isinstance(x, DataLoader)

def isdl(x) -> TypeGuard[ptdataloader]:
    '''Check if `x` is a `torch.utils.data.DataLoader`
    
    See Also
    --------
    isdataloader : Alias for `isdl`
    '''
    return isdataloader(x)

def isdatamodule(x) -> TypeGuard[pldatamodule]:
    '''Check if `x` is a `pytorch_lightning.LightningDataModule`
    
    See Also
    --------
    isdm : Alias for `isdatamodule`    
    '''
    return not isnone(x) and isinstance(x, DataModule)

def isdm(x) -> TypeGuard[pldatamodule]:
    '''Check if `x` is a `torch.utils.data.DataLoader`
    
    See Also
    --------
    isdatamodule : Alias for `isdm`
    '''
    return isdatamodule(x)


def isdfds(x) -> TypeGuard[ptdataset]:
    '''Check if `x` is a "DataFrame Dataset" i.e. a `torch.utils.data.Dataset` 
    with a `pd.DataFrame` as `df` attribute.

    See Also
    --------
    isdfdataset : Alias for `isdfds`
    isdataframedataset : Alias for `isdfds`
    '''
    return isds(x) and hasattr(x, 'df') and isdf(getattr(x, 'df', None))


def isdfdataset(x) -> TypeGuard[ptdataset]:
    '''Check if `x` is a "DataFrame Dataset" i.e. a `torch.utils.data.Dataset` 
    with a `pd.DataFrame` as `df` attribute.

    See Also
    --------
    isdfds : Alias for `isdfdataset`
    isdataframedataset : Alias for `isdfdataset`
    '''
    return isdfds(x)

def isdataframedataset(x) -> TypeGuard[ptdataset]:
    '''Check if `x` is a "DataFrame Dataset" i.e. a `torch.utils.data.Dataset` 
    with a `pd.DataFrame` as `df` attribute.

    See Also
    --------
    isdfdataset : Alias for `isdataframedataset`
    isdfds : Alias for `isdataframedataset`
    '''
    return isdfds(x)

# %% ../nbs/63_trch.ipynb 19
#| export



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/64_iscv.ipynb.

# %% auto 0
__all__ = ['isvideocapture', 'isvid', 'isvideo']

# %% ../nbs/64_iscv.ipynb 4
from typing import TypeGuard

# %% ../nbs/64_iscv.ipynb 5
from .atyp import VideoCapture, cvvideo

# %% ../nbs/64_iscv.ipynb 7
def isvideocapture(x) -> TypeGuard[cvvideo]:
    '''Check if `x` is a `cv2.VideoCapture`
    
     See Also
    --------
    isvid : Alias for `isvideocapture`
    isvideo : Alias for `isvideocapture`
    '''
    return isinstance(x, VideoCapture)

def isvid(x) -> TypeGuard[cvvideo]:
    '''Check if `x` is a `cv2.VideoCapture`
    
     See Also
    --------
    isvideo : Alias for `isvid`
    isvideocapture : Alias for `isvid`
    '''
    return isvideocapture(x)

def isvideo(x) -> TypeGuard[cvvideo]:
    '''Check if `x` is a `cv2.VideoCapture`
    
     See Also
    --------
    isvid : Alias for `isvideo`
    isvideocapture : Alias for `isvideo`
    '''
    return isvideocapture(x)
